/*
    bmark.c
    Copyright 1999 by Richard Dawe

    Simple Mailslot benchmarking for the "raw" functions and via DJGPP's
    File System Extension.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>

#include <mailslot.h>

#define NO_TRIALS 1000

int main (int argc, char *argv[])
{
    uclock_t start, finish;
    int fdWrite, fdRead, hMailslot;
    char data[MAILSLOT_MAXSIZE], buf[MAILSLOT_MAXSIZE];
    int wlen, rlen, tlen, no_successes, no_failures;
    float trate, interval;
    int i, j;

    /* Set up mailslot handling */
    __mailslot_install_handler();

    /* Set up the data buffers */
    for (i = 0; i < sizeof(data); i++) { data[i] = rand() % 255; }

    printf("--- Mailslot function benchmarking ---\n");
    start = uclock();

    hMailslot = __mailslot_create("\\mailslot\\bmark", MAILSLOT_MAXSIZE);
    if (hMailslot == -1) { perror("bmark"); return(EXIT_FAILURE); }

    for (j = no_successes = no_failures = tlen = 0; j < NO_TRIALS; j++) {
        if ((j % 100) == 0) {
            printf("Trial %i\r", j);
            fflush(stdout);
        }

        wlen = __mailslot_write("\\mailslot\\bmark",  data, sizeof(data), 0);
        rlen = __mailslot_read(hMailslot, buf,  sizeof(buf),  0);

        if (wlen == rlen) {
            if (memcmp(data, buf, rlen) == 0) {
                no_successes++;
                tlen += wlen;
            }
        } else {
            no_failures++;
        }
    }

    __mailslot_destroy(hMailslot);

    if (no_failures == 0) printf("+ All transfers successful\n");
    printf(". %i successful transfers, %i failed transfers\n",
           no_successes, no_failures);

    finish = uclock();

    interval = (float) ((finish - start) / UCLOCKS_PER_SEC);
    trate = (interval == 0) ? 0 : (float) (tlen / interval);
    printf(". Transfer rate = %f B/sec\n", trate);
    printf(". Transfer rate = %f KB/sec\n", trate / 1024);
    printf(". Transfer rate = %f MB/sec\n", trate / (1024 * 1024));

    printf("\n--- FSEXT benchmarking ---\n");
    start = uclock();

    fdRead  = open("/dev/mailslot/localhost/bmark", O_BINARY | O_RDONLY, S_IRUSR);
    if (fdRead == -1) { perror("bmark"); return(EXIT_FAILURE); }
    fdWrite = open("/dev/mailslot/localhost/bmark", O_BINARY | O_WRONLY, S_IWUSR);
    if (fdWrite == -1) { perror("bmark"); return(EXIT_FAILURE); }

    for (j = no_successes = no_failures = tlen = 0; j < NO_TRIALS; j++) {
        if ((j % 100) == 0) {
            printf("Trial %i\r", j);
            fflush(stdout);
        }

        wlen = write(fdWrite, data, sizeof(data));
        rlen = read(fdRead, buf, sizeof(buf));

        if (wlen == rlen) {
            if (memcmp(data, buf, rlen) == 0) {
                no_successes++;
                tlen += wlen;
            }
        } else {
            no_failures++;
        }
    }

    close(fdWrite);
    close(fdRead);

    if (no_failures == 0) printf("+ All transfers successful\n");
    printf(". %i successful transfers, %i failed transfers\n",
           no_successes, no_failures);

    finish = uclock();

    interval = (float) ((finish - start) / UCLOCKS_PER_SEC);
    trate = (interval == 0) ? 0 : (float) (tlen / interval);
    printf(". Transfer rate = %f B/sec\n", trate);
    printf(". Transfer rate = %f KB/sec\n", trate / 1024);
    printf(". Transfer rate = %f MB/sec\n", trate / (1024 * 1024));

    return(EXIT_SUCCESS);
}
