/*
    test.c
    Copyright 1999 by Richard Dawe

    This file tests the "raw" mailslot functions and the DJGPP File System
    Extension module.
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>

#include <mailslot.h>

#define TESTMSG    "hello"
#define MYCOMPUTER "anna95"

int main (int argc, char *argv[])
{
    int hMailslot, fdRead, fdWrite, ret;
    char buffer[256] = "";

    printf("--- Mailslot function tests ---\n\n");

    printf("- Local tests -\n");

    hMailslot = __mailslot_create("\\mailslot\\inetd", MAILSLOT_MAXSIZE);
    printf("Creation returned:    %i\n", hMailslot);

    ret = __mailslot_write("\\mailslot\\inetd", TESTMSG, strlen(TESTMSG) + 1, 0);
    printf("Write returned:       %i\n", ret);

    ret = __mailslot_peek(hMailslot, buffer, sizeof(buffer));
    if (ret != -1) buffer[ret] = '\0'; else buffer[0] = '\0';
    printf("Peek returned:        %i (Message: '%s')\n", ret, buffer);
    ret = __mailslot_read(hMailslot, buffer, sizeof(buffer), 0);
    if (ret != -1) buffer[ret] = '\0'; else buffer[0] = '\0';
    printf("Read returned:        %i (Message: '%s')\n", ret, buffer);
    ret = __mailslot_read(hMailslot, buffer, sizeof(buffer), 0);
    if (ret != -1) buffer[ret] = '\0'; else buffer[0] = '\0';
    printf("Read returned:        %i (Message: '%s')\n", ret, buffer);

    ret = __mailslot_destroy(hMailslot);
    printf("Destruction returned: %i\n", ret);

    printf("\n- Non-local tests -\n");

    hMailslot = __mailslot_create("\\mailslot\\inetd", MAILSLOT_MAXSIZE);
    printf("Creation returned:    %i\n", hMailslot);

    ret = __mailslot_write("\\\\*\\mailslot\\inetd", TESTMSG, strlen(TESTMSG) + 1, 0);
    printf("Write to * returned:  %i\n", ret);
    ret = __mailslot_read(hMailslot, buffer, sizeof(buffer), 0);
    if (ret != -1) buffer[ret] = '\0'; else buffer[0] = '\0';
    printf("Read returned:        %i (Message: '%s')\n", ret, buffer);
    ret = __mailslot_read(hMailslot, buffer, sizeof(buffer), 0);
    if (ret != -1) buffer[ret] = '\0'; else buffer[0] = '\0';
    printf("Read returned:        %i (Message: '%s')\n", ret, buffer);

    ret = __mailslot_write("\\\\" MYCOMPUTER "\\mailslot\\inetd",
                           TESTMSG, strlen(TESTMSG) + 1, 0);
    printf("Write remote returned:%i\n", ret);
    ret = __mailslot_read(hMailslot, buffer, sizeof(buffer), 0);
    if (ret != -1) buffer[ret] = '\0'; else buffer[0] = '\0';
    printf("Read returned:        %i (Message: '%s')\n", ret, buffer);
    ret = __mailslot_read(hMailslot, buffer, sizeof(buffer), 0);
    if (ret != -1) buffer[ret] = '\0'; else buffer[0] = '\0';
    printf("Read returned:        %i (Message: '%s')\n", ret, buffer);

    ret = __mailslot_destroy(hMailslot);
    printf("Destruction returned: %i\n", ret);
  
    printf("\n--- File System Extension tests ---\n\n");
    __mailslot_install_handler();

    fdRead = open("/dev/mailslot/localhost/woohoo",  O_BINARY | O_RDONLY, S_IRUSR);
    printf("Mailslot read fd:     %i\n", fdRead);
    fdWrite = open("/dev/mailslot/localhost/woohoo", O_BINARY | O_WRONLY, S_IWUSR);
    printf("Mailslot write fd:    %i\n", fdWrite);

    ret = write(fdWrite, TESTMSG, strlen(TESTMSG) + 1);
    printf("Write returned:       %i\n", ret);
    ret = read(fdRead, buffer, sizeof(buffer));
    if (ret != -1) buffer[ret] = '\0'; else buffer[0] = '\0';
    printf("Read returned:        %i (Message: '%s')\n", ret, buffer);
    ret = read(fdRead, buffer, sizeof(buffer));
    if (ret != -1) buffer[ret] = '\0'; else buffer[0] = '\0';
    printf("Read returned:        %i (Message: '%s')\n", ret, buffer);

    close(fdRead);
    close(fdWrite);

    return(EXIT_SUCCESS);
}
